@@ -190,6 +190,7 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 
 	timeout := x.Timeout
 	withContextDeadline := false
+sendRetry:
 	for retries := 0; ; retries++ {
 		if retries > 0 {
 			if x.OnRetry != nil {
@@ -202,6 +203,9 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 				break
 			}
 			if retries > x.Retries {
+				if err == nil {
+					err = fmt.Errorf("max retries (%d) exceeded", x.Retries)
+				}
 				if strings.Contains(err.Error(), "timeout") {
 					err = fmt.Errorf("request timeout (after %d retries)", retries-1)
 				}
@@ -293,15 +297,12 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 			var resp []byte
 			resp, err = x.receive()
 			if err == io.EOF && strings.HasPrefix(x.Transport, tcp) {
-				// EOF on TCP: reconnect and retry. Do not count
-				// as retry as socket was broken
 				x.Logger.Printf("ERROR: EOF. Performing reconnect")
 				err = x.netConnect()
 				if err != nil {
 					return nil, err
 				}
-				retries--
-				break
+				continue sendRetry
 			} else if err != nil {
 				// receive error. retrying won't help. abort
 				break
@@ -522,7 +523,7 @@ func (packet *SnmpPacket) marshalMsg() ([]byte, error) {
 		}
 	} else {
 		// community
-		buf.Write([]byte{4, uint8(len(packet.Community))})
+		buf.Write([]byte{4, uint8(len(packet.Community))}) //nolint:gosec
 		buf.WriteString(packet.Community)
 		// pdu
 		pdu, err2 := packet.marshalPDU()
@@ -704,7 +705,8 @@ func (packet *SnmpPacket) marshalPDU() ([]byte, error) {
 func (packet *SnmpPacket) marshalVBL() ([]byte, error) {
 	vblBuf := new(bytes.Buffer)
 	for _, pdu := range packet.Variables {
-		pdu := pdu
+		// The copy of the 'for' variable "pdu" can be deleted (Go 1.22+)
+		pdu := pdu //nolint:copyloopvar
 		vb, err := marshalVarbind(&pdu)
 		if err != nil {
 			return nil, err
@@ -942,7 +944,10 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
 		tmpBuf.Write(oid)
 		tmpBuf.WriteByte(byte(pdu.Type))
-		intBytes := marshalUint64(pdu.Value)
+		intBytes, err := marshalUint64(pdu.Value)
+		if err != nil {
+			return nil, fmt.Errorf("error marshalling: %w", err)
+		}
 		tmpBuf.WriteByte(byte(len(intBytes)))
 		tmpBuf.Write(intBytes)
 		tmpBytes := tmpBuf.Bytes()
@@ -979,12 +984,12 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 
 // -- Unmarshalling Logic ------------------------------------------------------
 
-func (x *GoSNMP) unmarshalHeader(packet []byte, response *SnmpPacket) (int, error) {
+func (x *GoSNMP) unmarshalVersionFromHeader(packet []byte, response *SnmpPacket) (SnmpVersion, int, error) {
 	if len(packet) < 2 {
-		return 0, fmt.Errorf("cannot unmarshal empty packet")
+		return 0, 0, fmt.Errorf("cannot unmarshal empty packet")
 	}
 	if response == nil {
-		return 0, fmt.Errorf("cannot unmarshal response into nil packet reference")
+		return 0, 0, fmt.Errorf("cannot unmarshal response into nil packet reference")
 	}
 
 	response.Variables = make([]SnmpPDU, 0, 5)
@@ -994,33 +999,42 @@ func (x *GoSNMP) unmarshalHeader(packet []byte, response *SnmpPacket) (int, erro
 
 	// First bytes should be 0x30
 	if PDUType(packet[0]) != Sequence {
-		return 0, fmt.Errorf("invalid packet header")
+		return 0, 0, fmt.Errorf("invalid packet header")
 	}
 
 	length, cursor, err := parseLength(packet)
 	if err != nil {
-		return 0, err
+		return 0, 0, err
 	}
 	if len(packet) != length {
-		return 0, fmt.Errorf("error verifying packet sanity: Got %d Expected: %d", len(packet), length)
+		return 0, 0, fmt.Errorf("error verifying packet sanity: Got %d Expected: %d", len(packet), length)
 	}
 	x.Logger.Printf("Packet sanity verified, we got all the bytes (%d)", length)
 
 	// Parse SNMP Version
 	rawVersion, count, err := parseRawField(x.Logger, packet[cursor:], "version")
 	if err != nil {
-		return 0, fmt.Errorf("error parsing SNMP packet version: %w", err)
+		return 0, 0, fmt.Errorf("error parsing SNMP packet version: %w", err)
 	}
 
 	cursor += count
 	if cursor >= len(packet) {
-		return 0, fmt.Errorf("error parsing SNMP packet, packet length %d cursor %d", len(packet), cursor)
+		return 0, 0, fmt.Errorf("error parsing SNMP packet, packet length %d cursor %d", len(packet), cursor)
 	}
 
 	if version, ok := rawVersion.(int); ok {
-		response.Version = SnmpVersion(version)
 		x.Logger.Printf("Parsed version %d", version)
+		return SnmpVersion(version), cursor, nil //nolint:gosec
 	}
+	return 0, cursor, err
+}
+
+func (x *GoSNMP) unmarshalHeader(packet []byte, response *SnmpPacket) (int, error) {
+	version, cursor, err := x.unmarshalVersionFromHeader(packet, response)
+	if err != nil {
+		return 0, err
+	}
+	response.Version = version
 
 	if response.Version == Version3 {
 		oldcursor := cursor
@@ -1106,7 +1120,7 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 	}
 
 	if requestid, ok := rawRequestID.(int); ok {
-		response.RequestID = uint32(requestid)
+		response.RequestID = uint32(requestid) //nolint:gosec
 		x.Logger.Printf("requestID: %d", response.RequestID)
 	}
 
@@ -1122,7 +1136,7 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 		}
 
 		if nonRepeaters, ok := rawNonRepeaters.(int); ok {
-			response.NonRepeaters = uint8(nonRepeaters)
+			response.NonRepeaters = uint8(nonRepeaters) //nolint:gosec
 		}
 
 		// Parse Max Repetitions
@@ -1136,7 +1150,7 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 		}
 
 		if maxRepetitions, ok := rawMaxRepetitions.(int); ok {
-			response.MaxRepetitions = uint32(maxRepetitions & 0x7FFFFFFF)
+			response.MaxRepetitions = uint32(maxRepetitions & 0x7FFFFFFF) //nolint:gosec
 		}
 	} else {
 		// Parse Error-Status
@@ -1150,8 +1164,8 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 		}
 
 		if errorStatus, ok := rawError.(int); ok {
-			response.Error = SNMPError(errorStatus)
-			x.Logger.Printf("errorStatus: %d", uint8(errorStatus))
+			response.Error = SNMPError(errorStatus)                //nolint:gosec
+			x.Logger.Printf("errorStatus: %d", uint8(errorStatus)) //nolint:gosec
 		}
 
 		// Parse Error-Index
@@ -1165,8 +1179,8 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 		}
 
 		if errorindex, ok := rawErrorIndex.(int); ok {
-			response.ErrorIndex = uint8(errorindex)
-			x.Logger.Printf("error-index: %d", uint8(errorindex))
+			response.ErrorIndex = uint8(errorindex)               //nolint:gosec
+			x.Logger.Printf("error-index: %d", uint8(errorindex)) //nolint:gosec
 		}
 	}
 
