@@ -10,6 +10,7 @@ import (
 	"encoding/base64"
 	"errors"
 	"fmt"
+	"io"
 	"os"
 
 	"github.com/gosnmp/gosnmp"
@@ -20,21 +21,20 @@ func isPlayback() bool {
 }
 
 func netSnmpPduPkt(fname string, _ gosnmp.SnmpPDU, _ *gosnmp.GoSNMP, _ uint32, _ bool) ([]byte, error) {
-	pberr := "error with net-snmp playback file, run test with `-rec=true` to create missing playback files: %w"
-
-	// find and return the capture file, else error
-	b64, err := os.ReadFile(fname)
+	f, err := os.Open(fname)
 	if err != nil {
-		return nil, fmt.Errorf(pberr, err)
+		if os.IsNotExist(err) {
+			return nil, fmt.Errorf("%w; run test with `-rec=true` to create missing playback files", err)
+		}
+		return nil, err
 	}
+	defer f.Close()
 
-	out, err := base64.StdEncoding.DecodeString(string(b64))
+	out, err := io.ReadAll(base64.NewDecoder(base64.StdEncoding, f))
 	if err != nil {
-		return nil, fmt.Errorf(pberr, err)
-	}
-
-	if len(out) == 0 {
-		return nil, fmt.Errorf(pberr, errors.New("empty file"))
+		return nil, err
+	} else if len(out) == 0 {
+		return nil, errors.New("netsnmp playback is empty")
 	}
 
 	return out, nil
